@@ -576,6 +576,7 @@ def cert_builder(
 ):
     key = key or DEF_KEY_SIZE
     extensions = [parse_extension(e) for e in extensions] if extensions else []
+    purpose = purpose or CertPurpose._
 
     if isinstance(key, int):
         _ret_key = True
@@ -624,7 +625,7 @@ def cert_builder(
     ext_key_usage = ext_key_usage or []
 
     if CertPurpose.CA in purpose:
-        key_usage = {"key_cert_sign": True, "crl_sign": True}
+        key_usage.update({"key_cert_sign": True, "crl_sign": True})
         builder = builder.add_extension(x509.BasicConstraints(True, 0), critical=True)
 
     if CertPurpose.SERVER in purpose:
